@@ -9,6 +9,7 @@ import toml
 import warnings
 import time
 from datetime import timedelta
+import tensorflow as tf
 import tensorflow.keras as ks
 
 import orcanet.backend as backend
@@ -17,6 +18,7 @@ from orcanet.in_out import IOHandler
 from orcanet.history import HistoryHandler
 from orcanet.utilities.nn_utilities import load_zero_center_data, get_auto_label_modifier
 import orcanet.logging as olog
+import medgeconv
 
 
 class Organizer:
@@ -75,8 +77,9 @@ class Organizer:
         self.xs_mean = None
         self._auto_label_modifier = None
         self._stored_model = None
+        self._strategy = None
 
-    def train_and_validate(self, model=None, epochs=None):
+    def train_and_validate(self, model=None, epochs=None, to_epoch=None):
         """
         Train a model and validate according to schedule.
 
@@ -102,6 +105,9 @@ class Organizer:
             None for infinite. This includes the current epoch in case it
             is not finished yet, i.e. 1 means complete the epoch if there
             are files left, otherwise do the next epoch.
+        to_epoch : int, optional
+            Train up to and including this epoch. Can not be used together with
+            epochs.
 
         Returns
         -------
@@ -123,8 +129,19 @@ class Organizer:
         next_epoch = self.io.get_next_epoch(latest_epoch)
         n_train_files = self.io.get_no_of_files("train")
 
+        if to_epoch is None:
+            epochs_left = epochs
+        else:
+            if epochs is not None:
+                raise ValueError("Can not give both 'epochs' and 'to_epoch'")
+            if latest_epoch is None:
+                epochs_left = to_epoch
+            else:
+                epochs_left = max(
+                    0, to_epoch - self.io.get_next_epoch(latest_epoch)[0] + 1)
+
         trained_epochs = 0
-        while epochs is None or trained_epochs < epochs:
+        while epochs_left is None or trained_epochs < epochs_left:
             # Train on remaining files
             for file_no in range(next_epoch[1], n_train_files + 1):
                 curr_epoch = (next_epoch[0], file_no)
@@ -271,9 +288,6 @@ class Organizer:
         smry_logger.write_line(epoch_float, "n/a", history_val=history)
 
         update_summary_plot(self)
-        if self.cfg.make_weight_plots:
-            backend.save_actv_wghts_plot(
-                self, model, latest_epoch, samples=self.cfg.batchsize)
 
         if self.cfg.cleanup_models:
             self.cleanup_models()
@@ -367,9 +381,9 @@ class Organizer:
         Parameters
         ----------
         epoch : int, optional
-            Epoch of a model to load.
+            Epoch of a model to load [default: lowest val loss].
         fileno : int, optional
-            File number of a model to load.
+            File number of a model to load [default: lowest val loss].
 
         Returns
         -------
@@ -513,7 +527,8 @@ class Organizer:
         Parameters
         ----------
         epoch : int
-            Epoch of the saved model.
+            Epoch of the saved model. If both this and fileno are -1,
+            load the most recent model.
         fileno : int
             Fileno of the saved model.
         logging : bool
@@ -527,9 +542,7 @@ class Organizer:
         path_of_model = self.io.get_model_path(epoch, fileno)
         path_loc = self.io.get_model_path(epoch, fileno, local=True)
         self.io.print_log("Loading saved model: " + path_loc, logging=logging)
-        model = ks.models.load_model(
-            path_of_model, custom_objects=self.cfg.custom_objects)
-        return model
+        return self._load_model(path_of_model)
 
     def _get_model(self, model, logging=False):
         """ Load most recent saved model or use user model. """
@@ -544,7 +557,7 @@ class Organizer:
             elif isinstance(model, str):
                 # path to a saved model
                 self.io.print_log("Loading model from " + model, logging=logging)
-                model = ks.models.load_model(model)
+                model = self._load_model(model)
 
             if logging:
                 self._save_as_json(model)
@@ -566,10 +579,17 @@ class Organizer:
             elif isinstance(model, str):
                 # path to a saved model
                 self.io.print_log("Loading model from " + model, logging=logging)
-                model = ks.models.load_model(model)
+                model = self._load_model(model)
 
         return model
 
+    def _load_model(self, filepath):
+        """ Load from path, with custom objects and parallized. """
+        with self.get_strategy().scope():
+            model = ks.models.load_model(
+                filepath, custom_objects=self.cfg.get_custom_objects())
+        return model
+
     def _save_as_json(self, model):
         """ Save the architecture of a model as json to fixed path. """
         json_filename = "model_arch.json"
@@ -605,6 +625,17 @@ class Organizer:
                      epoch[1] % self.cfg.validate_interval == 0)
         return val_sched
 
+    def get_strategy(self):
+        """ Get the strategy for distributed training. """
+        if self._strategy is None:
+            if self.cfg.multi_gpu and len(
+                    tf.config.list_physical_devices('GPU')) > 1:
+                self._strategy = tf.distribute.MirroredStrategy()
+            else:
+                self._strategy = tf.distribute.get_strategy()
+            print(f'Number of GPUs: {self._strategy.num_replicas_in_sync}')
+        return self._strategy
+
 
 class Configuration(object):
     """
@@ -661,16 +692,17 @@ class Configuration(object):
         learning rate per file (e.g. 0.1 for 10% decrease per file).
         If it is a function: Takes as an input the epoch and the
         file number (in this order), and returns the learning rate.
+        Both epoch and fileno start at 1, i.e. 1, 1 is the start of the
+        training.
         If it is a str: Path to a csv file inside the main folder, containing
         3 columns with the epoch, fileno, and the value the lr will be set
         to when reaching this epoch/fileno.
-    make_weight_plots : bool
-        If true, generate and save plots of the activations and weights of
-        the network to the 'plots/' subfolder.
     max_queue_size : int
         max_queue_size option of the keras training and evaluation generator
         methods. How many batches get preloaded
         from the generator.
+    multi_gpu : bool
+        Use all availble GPUs (distributed training if theres more then one).
     n_events : None or int
         For testing purposes. If not the whole .h5 file should be used for
         training, define the number of samples.
@@ -754,11 +786,12 @@ class Configuration(object):
         self.use_scratch_ssd = False
         self.verbose_train = 1
         self.verbose_val = 0
-        self.make_weight_plots = True
+        self.make_weight_plots = False  # Removed in v0.11.1
         self.n_events = None
         self.max_queue_size = 10
         self.train_logger_display = 100
         self.train_logger_flush = -1
+        self.multi_gpu = True
 
         self._default_values = dict(self.__dict__)
 
@@ -790,6 +823,10 @@ class Configuration(object):
                 raise AttributeError(
                     "Unknown attribute {}".format(key))
 
+        # deprecation warning TODO remove in the future
+        if self.make_weight_plots:
+            warnings.warn("make_weight_plots was removed in version v0.11.1")
+
     def import_list_file(self, list_file):
         """
         Import the filepaths of the h5 files from a toml list file.
@@ -871,20 +908,10 @@ class Configuration(object):
             raise AttributeError("No {} files have been specified!".format(which))
         return self._files_dict[which]
 
-    @property
-    def default_values(self):
-        """ The default values for all settings. """
-        return self._default_values
-
-    @property
-    def key_samples(self):
-        """ Backward compatibility """
-        return self.key_x_values
-
-    @property
-    def key_labels(self):
-        """ Backward compatibility """
-        return self.key_y_values
+    def get_custom_objects(self):
+        """ Get user custom objects + orcanet internal ones. """
+        orcanet_co = medgeconv.custom_objects
+        return {**orcanet_co, **self.custom_objects}
 
 
 def _get_h5_files(folder):
